@@ -44,13 +44,17 @@ using Real = float;
 namespace py = pybind11;
 using std::swap;
 
-template <class T, class U> using pair = std::pair<T, U>;
+template <class T, class U>
+using pair = std::pair<T, U>;
 
-template <class T> using vec = std::vector<T>;
+template <class T>
+using vec = std::vector<T>;
 
-template <class T> using deque = std::deque<T>;
+template <class T>
+using deque = std::deque<T>;
 
-template <class T> using queue = std::queue<T, deque<T>>;
+template <class T>
+using queue = std::queue<T, deque<T>>;
 
 static std::mt19937 rand_src(42);
 static const float REBUILD_THRE = 1.5;
@@ -63,42 +67,52 @@ static const float REBUILD_THRE = 1.5;
 #define unlikely(x) (x)
 #endif
 
-
-std::string compress(std::string& data){
+std::string compress(std::string &data)
+{
   std::string output;
   snappy::Compress(data.data(), data.size(), &output);
   return output;
 }
 
-std::string decompress(std::string& data){
+std::string decompress(std::string &data)
+{
   std::string output;
   snappy::Uncompress(data.data(), data.size(), &output);
   return output;
 }
 
-template <typename Iter> Iter select_randomly(Iter start, Iter end) {
+template <typename Iter>
+Iter select_randomly(Iter start, Iter end)
+{
   std::uniform_int_distribution<> dis(0, std::distance(start, end) - 1);
   std::advance(start, dis(rand_src));
   return start;
 };
 
-template <int D = 2> class BB {
+template <int D = 2>
+class BB
+{
 private:
   std::array<Real, 2 * D> values;
 
 public:
   BB() { clear(); }
 
-  template <typename Iter> BB(const Iter &minima, const Iter &maxima) {
+  template <typename Iter>
+  BB(const Iter &minima, const Iter &maxima)
+  {
     std::array<Real, 2 * D> v;
-    if (unlikely(minima.size() != maxima.size())) {
+    if (unlikely(minima.size() != maxima.size()))
+    {
       throw std::runtime_error("Invalid size");
     }
     const int n = minima.size();
-    if (unlikely(n != D)) {
+    if (unlikely(n != D))
+    {
       throw std::runtime_error("Invalid size");
     }
-    for (int i = 0; i < n; ++i) {
+    for (int i = 0; i < n; ++i)
+    {
       v[i] = -minima[i];
       v[i + D] = maxima[i];
     }
@@ -106,152 +120,192 @@ public:
     values = v;
   }
 
-  BB(const std::array<Real, 2 * D> v) {
+  BB(const std::array<Real, 2 * D> v)
+  {
     validate(v);
     values = v;
   }
 
-  Real min(const int dim) const {
-    if (unlikely(dim < 0 || D <= dim)) {
+  Real min(const int dim) const
+  {
+    if (unlikely(dim < 0 || D <= dim))
+    {
       throw std::runtime_error("Invalid dim");
     }
     return -values[dim];
   }
-  Real max(const int dim) const {
-    if (unlikely(dim < 0 || D <= dim)) {
+  Real max(const int dim) const
+  {
+    if (unlikely(dim < 0 || D <= dim))
+    {
       throw std::runtime_error("Invalid dim");
     }
     return values[dim + D];
   }
 
-  bool validate(const std::array<Real, 2 * D> &v) const {
+  bool validate(const std::array<Real, 2 * D> &v) const
+  {
     bool flag = false;
-    for (int i = 0; i < 2; ++i) {
-      if (-v[i] > v[i + D]) {
+    for (int i = 0; i < 2; ++i)
+    {
+      if (-v[i] > v[i + D])
+      {
         flag = true;
         break;
       }
     }
-    if (flag) {
+    if (flag)
+    {
       throw std::runtime_error("Invalid Bounding Box");
     }
     return flag;
   }
   void clear() { std::fill_n(values.begin(), 2 * D, -1e100); }
 
-  BB operator+(const BB &rhs) const {
+  BB operator+(const BB &rhs) const
+  {
     std::array<Real, 2 * D> result;
-    for (int i = 0; i < 2 * D; ++i) {
+    for (int i = 0; i < 2 * D; ++i)
+    {
       result[i] = std::max(values[i], rhs.values[i]);
     }
     return BB<D>(result);
   }
 
-  BB operator+=(const BB &rhs) {
-    for (int i = 0; i < 2 * D; ++i) {
+  BB operator+=(const BB &rhs)
+  {
+    for (int i = 0; i < 2 * D; ++i)
+    {
       values[i] = std::max(values[i], rhs.values[i]);
     }
     return *this;
   }
 
-  void expand(const std::array<Real, D> &delta) {
-    for (int i = 0; i < D; ++i) {
+  void expand(const std::array<Real, D> &delta)
+  {
+    for (int i = 0; i < D; ++i)
+    {
       values[i] += delta[i];
       values[i + D] += delta[i];
     }
   }
 
   bool operator()(
-      const BB &target) const { // whether this and target has any intersect
+      const BB &target) const
+  { // whether this and target has any intersect
     std::array<Real, 2 * D> result;
-    for (int i = 0; i < 2 * D; ++i) {
+    for (int i = 0; i < 2 * D; ++i)
+    {
       result[i] = std::min(values[i], target.values[i]);
     }
     bool flag = true;
-    for (int i = 0; i < D; ++i) {
+    for (int i = 0; i < D; ++i)
+    {
       flag = flag && (-result[i] <= result[i + D]);
     }
     return flag;
   }
 
-  Real area() const {
+  Real area() const
+  {
     Real result = 1;
-    for (int i = 0; i < D; ++i) {
+    for (int i = 0; i < D; ++i)
+    {
       result *= values[i + D] - values[i];
     }
     return result;
   }
 
   Real operator[](const int i) const { return values[i]; }
-  template <class Archive> void serialize(Archive &ar) { ar(values); }
+  template <class Archive>
+  void serialize(Archive &ar) { ar(values); }
 };
 
-template <class T, int D = 2> class DataType {
+template <class T, int D = 2>
+class DataType
+{
 public:
   T first;
   BB<D> second;
   DataType(){};
 
-  DataType(const T &f, const BB<D> &s) {
+  DataType(const T &f, const BB<D> &s)
+  {
     first = f;
     second = s;
   }
 
-  DataType(T &&f, BB<D> &&s) noexcept {
+  DataType(T &&f, BB<D> &&s) noexcept
+  {
     first = std::move(f);
     second = std::move(s);
   }
 
-  template <class Archive> void serialize(Archive &ar) { ar(first, second); }
+  template <class Archive>
+  void serialize(Archive &ar) { ar(first, second); }
 };
 
-template <class T, int B = 6, int D = 2> class Leaf {
+template <class T, int B = 6, int D = 2>
+class Leaf
+{
 public:
   int axis = 0;
   BB<D> mbb;
   vec<DataType<T, D>> data; // You can swap when filtering
   // T is type of keys(ids) which will be returned when you post a query.
-  Leaf() {
+  Leaf()
+  {
     mbb = BB<D>();
     data.reserve(B);
   }
-  Leaf(int &_axis) {
+  Leaf(int &_axis)
+  {
     axis = _axis;
     mbb = BB<D>();
     data.reserve(B);
   }
 
-  Real area() const {
+  Real area() const
+  {
     return mbb.area();
   }
 
-  template <class Archive> void serialize(Archive &ar) { ar(axis, mbb, data); }
+  template <class Archive>
+  void serialize(Archive &ar) { ar(axis, mbb, data); }
 
   void set_axis(const int &_axis) { axis = _axis; }
 
-  void push(const T &key, const BB<D> &target) {
+  void push(const T &key, const BB<D> &target)
+  {
     data.emplace_back(key, target);
     update_mbb();
   }
 
-  void update_mbb() {
+  void update_mbb()
+  {
     mbb.clear();
-    for (const auto &datum : data) {
+    for (const auto &datum : data)
+    {
       mbb += datum.second;
     }
   }
 
   template <typename F>
   bool filter(DataType<T, D> &value,
-              const F &comp) { // false means given value is ignored
+              const F &comp)
+  { // false means given value is ignored
     if (unlikely(data.size() <
-                 B)) { // if there is room, just push the candidate
+                 B))
+    { // if there is room, just push the candidate
       mbb += value.second;
       data.push_back(std::move(value));
       return true;
-    } else { // if there is no room, check the priority and swap if needed
+    }
+    else
+    { // if there is no room, check the priority and swap if needed
       auto iter = std::upper_bound(data.begin(), data.end(), value, comp);
-      if (unlikely(iter != data.end())) {
+      if (unlikely(iter != data.end()))
+      {
         swap(*iter, value);
         update_mbb();
       }
@@ -259,74 +313,94 @@ public:
     }
   }
 
-  void operator()(const BB<D> &target, vec<T> &out) const {
-    if (unlikely(mbb(target))) {
-      for (const auto &x : data) {
-        if (unlikely(x.second(target))) {
+  void operator()(const BB<D> &target, vec<T> &out) const
+  {
+    if (unlikely(mbb(target)))
+    {
+      for (const auto &x : data)
+      {
+        if (unlikely(x.second(target)))
+        {
           out.emplace_back(x.first);
         }
       }
     }
   }
 
-  void del(const T &key, const BB<D> &target) {
-    if (unlikely(mbb(target))) {
+  void del(const T &key, const BB<D> &target)
+  {
+    if (unlikely(mbb(target)))
+    {
       auto remove_it =
-          std::remove_if(data.begin(), data.end(), [&](auto &datum) {
-            return datum.second(target) && datum.first == key;
-          });
+          std::remove_if(data.begin(), data.end(), [&](auto &datum)
+                         { return datum.second(target) && datum.first == key; });
       data.erase(remove_it, data.end());
     }
   }
 };
 
-template <class T, int B = 6, int D = 2> class PseudoPRTreeNode {
+template <class T, int B = 6, int D = 2>
+class PseudoPRTreeNode
+{
 public:
   std::array<Leaf<T, B, D>, 2 * D> leaves;
   std::unique_ptr<PseudoPRTreeNode> left, right;
 
-  PseudoPRTreeNode() {
-    for (int i = 0; i < 2 * D; i++) {
+  PseudoPRTreeNode()
+  {
+    for (int i = 0; i < 2 * D; i++)
+    {
       leaves[i].set_axis(i);
     }
   }
 
-  template <class Archive> void serialize(Archive &archive) {
+  template <class Archive>
+  void serialize(Archive &archive)
+  {
     // archive(cereal::(left), cereal::defer(right), leaves);
     archive(left, right, leaves);
   }
 
-  void address_of_leaves(vec<Leaf<T, B, D> *> &out) {
-    for (auto &leaf : leaves) {
-      if (likely(leaf.data.size() > 0)) {
+  void address_of_leaves(vec<Leaf<T, B, D> *> &out)
+  {
+    for (auto &leaf : leaves)
+    {
+      if (likely(leaf.data.size() > 0))
+      {
         out.emplace_back(&leaf);
       }
     }
   }
 
-  template <class iterator> auto filter(iterator &b, iterator &e) {
+  template <class iterator>
+  auto filter(iterator &b, iterator &e)
+  {
     vec<std::function<bool(const DataType<T, D> &, const DataType<T, D> &)>>
         comps;
-    for (int axis = 0; axis < 2 * D; ++axis) {
+    for (int axis = 0; axis < 2 * D; ++axis)
+    {
       comps.emplace_back(
-          [axis](const DataType<T, D> &lhs, const DataType<T, D> &rhs) {
+          [axis](const DataType<T, D> &lhs, const DataType<T, D> &rhs)
+          {
             return lhs.second[axis] < rhs.second[axis];
           });
     }
 
-    auto out = std::remove_if(b, e, [&](auto &x) {
+    auto out = std::remove_if(b, e, [&](auto &x)
+                              {
       for (auto &l : leaves) {
         if (unlikely(l.filter(x, comps[l.axis]))) {
           return true;
         }
       }
-      return false;
-    });
+      return false; });
     return out;
   }
 };
 
-template <class T, int B = 6, int D = 2> class PseudoPRTree {
+template <class T, int B = 6, int D = 2>
+class PseudoPRTree
+{
 public:
   std::unique_ptr<PseudoPRTreeNode<T, B, D>> root;
   vec<Leaf<T, B, D> *> cache_children;
@@ -334,28 +408,36 @@ public:
 
   PseudoPRTree() { root = std::make_unique<PseudoPRTreeNode<T, B, D>>(); }
 
-  template <class iterator> PseudoPRTree(iterator &b, iterator &e) {
-    if (likely(!root)) {
+  template <class iterator>
+  PseudoPRTree(iterator &b, iterator &e)
+  {
+    if (likely(!root))
+    {
       root = std::make_unique<PseudoPRTreeNode<T, B, D>>();
     }
     construct(root.get(), b, e, 0);
 
-    for (DataType<T, D> *it = e - 1; it >= b; --it) {
+    for (DataType<T, D> *it = e - 1; it >= b; --it)
+    {
       it->~DataType<T, D>();
     }
     b = nullptr;
     e = nullptr;
   }
 
-  template <class Archive> void serialize(Archive &archive) {
+  template <class Archive>
+  void serialize(Archive &archive)
+  {
     archive(root);
     // archive.serializeDeferments();
   }
 
   template <class iterator>
   void construct(PseudoPRTreeNode<T, B, D> *node, iterator &b, iterator &e,
-                 const size_t depth) {
-    if (e - b > 0 && node != nullptr) {
+                 const size_t depth)
+  {
+    if (e - b > 0 && node != nullptr)
+    {
       bool use_recursive_threads = std::pow(2, depth + 1) <= nthreads;
 
       vec<std::thread> threads;
@@ -369,74 +451,99 @@ public:
       auto mm = m;
       std::nth_element(b, m, ee,
                        [axis](const DataType<T, D> &lhs,
-                              const DataType<T, D> &rhs) noexcept {
+                              const DataType<T, D> &rhs) noexcept
+                       {
                          return lhs.second[axis] < rhs.second[axis];
                        });
 
-      if (m - b > 0) {
+      if (m - b > 0)
+      {
         node->left = std::make_unique<PseudoPRTreeNode<T, B, D>>();
         node_left = node->left.get();
-        if (use_recursive_threads) {
+        if (use_recursive_threads)
+        {
           threads.push_back(
-              std::thread([&]() { construct(node_left, b, m, depth + 1); }));
-        } else {
+              std::thread([&]()
+                          { construct(node_left, b, m, depth + 1); }));
+        }
+        else
+        {
           construct(node_left, b, m, depth + 1);
         }
       }
-      if (ee - mm > 0) {
+      if (ee - mm > 0)
+      {
         node->right = std::make_unique<PseudoPRTreeNode<T, B, D>>();
         node_right = node->right.get();
-        if (use_recursive_threads) {
+        if (use_recursive_threads)
+        {
           threads.push_back(
-              std::thread([&]() { construct(node_right, mm, ee, depth + 1); }));
-        } else {
+              std::thread([&]()
+                          { construct(node_right, mm, ee, depth + 1); }));
+        }
+        else
+        {
           construct(node_right, mm, ee, depth + 1);
         }
       }
       std::for_each(threads.begin(), threads.end(),
-                    [&](std::thread &x) { x.join(); });
+                    [&](std::thread &x)
+                    { x.join(); });
       threads.clear();
       vec<std::thread>().swap(threads);
     }
   }
 
-  auto get_all_leaves(const int hint) {
-    if (cache_children.empty()) {
+  auto get_all_leaves(const int hint)
+  {
+    if (cache_children.empty())
+    {
       using U = PseudoPRTreeNode<T, B, D>;
+      vec<U *> leaf_nodes;
       cache_children.reserve(hint);
       auto node = root.get();
       queue<U *> que;
       que.emplace(node);
 
-      while (likely(!que.empty())) {
+      while (likely(!que.empty()))
+      {
         node = que.front();
         que.pop();
-        node->address_of_leaves(cache_children);
+        leaf_nodes.emplace_back(node);
         if (node->left)
           que.emplace(node->left.get());
         if (node->right)
           que.emplace(node->right.get());
       }
+
+      parallel_for_each(leaf_nodes.begin(), leaf_nodes.end(), cache_children,
+                        [&](auto &node, auto &o)
+                        {
+                          node->address_of_leaves(o);
+                        });
     }
     return cache_children;
   }
 
   std::pair<DataType<int, D> *, DataType<int, D> *> as_X(void *placement,
-                                                         const int hint) {
+                                                         const int hint)
+  {
     DataType<int, D> *b, *e;
     auto children = get_all_leaves(hint);
     int total = children.size();
     b = (DataType<int, D> *)placement;
     e = b + total;
-    parallel_for_each(b, e, [&](auto &p) {
+    parallel_for_each(b, e, [&](auto &p)
+                      {
       int i = &p - b;
-      new (b + i) DataType<int, D>{i, children[i]->mbb};
-    });
+      new (b + i) DataType<int, D>{i, children[i]->mbb}; });
     return {b, e};
   }
 };
 
-template <class T, int B = 6, int D = 2> class PRTreeNode {
+template <class T, int B = 6, int D = 2>
+class PRTreeNode
+{
 public:
   BB<D> mbb;
   std::unique_ptr<Leaf<T, B, D>> leaf;
@@ -447,7 +554,8 @@ public:
 
   PRTreeNode(BB<D> &&_mbb) { mbb = std::move(_mbb); }
 
-  PRTreeNode(Leaf<T, B, D> *l) {
+  PRTreeNode(Leaf<T, B, D> *l)
+  {
     leaf = std::make_unique<Leaf<T, B, D>>();
     mbb = l->mbb;
     leaf->mbb = std::move(l->mbb);
@@ -456,13 +564,17 @@ public:
 
   bool operator()(const BB<D> &target) { return mbb(target); }
 
-  template <class Archive> void serialize(Archive &archive) {
+  template <class Archive>
+  void serialize(Archive &archive)
+  {
     // archive(cereal::defer(leaf), cereal::defer(head), cereal::defer(next));
     archive(mbb, leaf, head, next);
   }
 };
 
-template <class T, int B = 6, int D = 2> class PRTree {
+template <class T, int B = 6, int D = 2>
+class PRTree
+{
 private:
   std::unique_ptr<PRTreeNode<T, B, D>> root;
   std::unordered_map<T, BB<D>> idx2bb;
@@ -471,12 +583,15 @@ private:
   std::atomic<T> global_idx = 0;
 
 public:
-  template <class Archive> void serialize(Archive &archive) {
+  template <class Archive>
+  void serialize(Archive &archive)
+  {
     archive(root, idx2bb, idx2data, global_idx, n_at_build);
     // archive.serializeDeferments();
   }
 
-  void save(std::string fname) {
+  void save(std::string fname)
+  {
     {
       {
         std::ofstream ofs(fname, std::ios::binary);
@@ -490,13 +605,15 @@ public:
     }
   }
 
-  PRTree() {
+  PRTree()
+  {
     root = std::make_unique<PRTreeNode<T, B, D>>();
   }
 
   PRTree(std::string fname) { load(fname); }
 
-  void load(std::string fname) {
+  void load(std::string fname)
+  {
     {
       {
         std::ifstream ifs(fname, std::ios::binary);
@@ -511,15 +628,19 @@ public:
     }
   }
 
-  PRTree(const py::array_t<T> &idx, const py::array_t<float> &x) {
+  PRTree(const py::array_t<T> &idx, const py::array_t<float> &x)
+  {
     const auto &buff_info_idx = idx.request();
     const auto &shape_idx = buff_info_idx.shape;
     const auto &buff_info_x = x.request();
     const auto &shape_x = buff_info_x.shape;
-    if (shape_idx[0] != shape_x[0]) {
+    if (shape_idx[0] != shape_x[0])
+    {
       throw std::runtime_error(
           "Both index and boudning box must have the same length");
-    } else if (shape_x[1] != 2 * D) {
+    }
+    else if (shape_x[1] != 2 * D)
+    {
       throw std::runtime_error(
           "Bounding box must have the shape (length, 2 * dim)");
     }
@@ -535,7 +656,8 @@ public:
     b = reinterpret_cast<DataType<T, D> *>(placement);
     e = b + length;
 
-    parallel_for_each(b, e, [&](auto &it) {
+    parallel_for_each(b, e, [&](auto &it)
+                      {
       int i = &it - b;
       std::array<Real, D> minima;
       std::array<Real, D> maxima;
@@ -544,13 +666,14 @@ public:
         maxima[j] = rx(i, j + D);
       }
       auto bb = BB<D>(minima, maxima);
-      new (b + i) DataType<T, D>{ri(i), std::move(bb)};
-    });
+      new (b + i) DataType<T, D>{ri(i), std::move(bb)}; });
 
-    for (size_t i = 0; i < length; i++) {
+    for (size_t i = 0; i < length; i++)
+    {
       std::array<Real, D> minima;
       std::array<Real, D> maxima;
-      for (int j = 0; j < D; ++j) {
+      for (int j = 0; j < D; ++j)
+      {
         minima[j] = rx(i, j);
         maxima[j] = rx(i, j + D);
       }
@@ -561,24 +684,29 @@ public:
     std::free(placement);
   }
 
-  void set_obj(const T &idx, std::optional<std::string> objdumps = std::nullopt){
-    if (objdumps){
+  void set_obj(const T &idx, std::optional<std::string> objdumps = std::nullopt)
+  {
+    if (objdumps)
+    {
       auto val = objdumps.value();
       idx2data.emplace(idx, compress(val));
     }
   }
 
-  py::object get_obj(const T &idx){
+  py::object get_obj(const T &idx)
+  {
     py::object obj = py::none();
     auto search = idx2data.find(idx);
-    if (likely(search != idx2data.end())){
+    if (likely(search != idx2data.end()))
+    {
       auto val = idx2data.at(idx);
       obj = py::cast<py::object>(py::bytes(decompress(val)));
     }
     return obj;
   }
 
-  void insert(const T &idx, const py::array_t<float> &x, const std::optional<std::string> objdumps = std::nullopt) {
+  void insert(const T &idx, const py::array_t<float> &x, const std::optional<std::string> objdumps = std::nullopt)
+  {
     vec<Leaf<T, B, D> *> cands;
     queue<PRTreeNode<T, B, D> *> que;
     std::stack<PRTreeNode<T, B, D> *> sta;
@@ -588,17 +716,20 @@ public:
     const auto &buff_info_x = x.request();
     const auto &shape_x = buff_info_x.shape;
     const auto &ndim = buff_info_x.ndim;
-    if (shape_x[0] != 2 * D || ndim != 1) {
+    if (shape_x[0] != 2 * D || ndim != 1)
+    {
       throw std::runtime_error("invalid shape.");
     }
     auto it = idx2bb.find(idx);
-    if (unlikely(it != idx2bb.end())) {
+    if (unlikely(it != idx2bb.end()))
+    {
       throw std::runtime_error("Given index is already included.");
     }
     {
       std::array<Real, D> minima;
       std::array<Real, D> maxima;
-      for (int i = 0; i < D; ++i) {
+      for (int i = 0; i < D; ++i)
+      {
         minima[i] = *x.data(i);
         maxima[i] = *x.data(i + D);
       }
@@ -608,22 +739,27 @@ public:
     set_obj(idx, objdumps);
 
     std::array<Real, D> delta;
-    for (int i = 0; i < D; ++i) {
+    for (int i = 0; i < D; ++i)
+    {
       delta[i] = bb.max(i) - bb.min(i) + 0.00000001;
     }
 
     // find the leaf node to insert
     Real c = 0.0;
-    auto qpush_if_intersect = [&](PRTreeNode<T, B, D> *r) {
-      if (unlikely((*r)(bb))) {
+    auto qpush_if_intersect = [&](PRTreeNode<T, B, D> *r)
+    {
+      if (unlikely((*r)(bb)))
+      {
         que.emplace(r);
         sta.push(r);
       }
     };
 
-    while (likely(cands.size() == 0)) {
+    while (likely(cands.size() == 0))
+    {
       std::array<Real, D> d;
-      for (int i = 0; i < D; ++i) {
+      for (int i = 0; i < D; ++i)
+      {
         d[i] = delta[i] * c;
       }
       bb.expand(d);
@@ -632,18 +768,24 @@ public:
       // depth first search
       p = root.get();
       qpush_if_intersect(p);
-      while (likely(!que.empty())) {
+      while (likely(!que.empty()))
+      {
         p = que.front();
         que.pop();
 
-        if (unlikely(p->leaf)) {
+        if (unlikely(p->leaf))
+        {
           // if p is leaf, then it is the leaf node to insert if it intersects
           cands.push_back(p->leaf.get());
-        } else {
-          if (likely(p->head)) {
+        }
+        else
+        {
+          if (likely(p->head))
+          {
             q = p->head.get();
             qpush_if_intersect(q);
-            while (likely(q->next)) {
+            while (likely(q->next))
+            {
               q = q->next.get();
               qpush_if_intersect(q);
             }
@@ -656,12 +798,14 @@ public:
     Leaf<T, B, D> *min_leaf = nullptr;
     {
       Real min_diff_area = 1e100;
-      for (const auto &leaf : cands) {
+      for (const auto &leaf : cands)
+      {
         Leaf<T, B, D> tmp_leaf = Leaf<T, B, D>(*leaf);
         Real diff_area = -tmp_leaf.area();
         tmp_leaf.push(idx, bb);
         diff_area += tmp_leaf.area();
-        if (min_leaf == nullptr || diff_area < min_diff_area) {
+        if (min_leaf == nullptr || diff_area < min_diff_area)
+        {
           min_diff_area = diff_area;
           min_leaf = leaf;
         }
@@ -669,16 +813,21 @@ public:
     }
     min_leaf->push(idx, bb);
     // update mbbs of all cands and their parents
-    while (likely(!sta.empty())) {
+    while (likely(!sta.empty()))
+    {
       p = sta.top();
       sta.pop();
-      if (unlikely(p->leaf)) {
+      if (unlikely(p->leaf))
+      {
         p->mbb = p->leaf->mbb;
-      } else if (likely(p->head)) {
+      }
+      else if (likely(p->head))
+      {
         BB<D> mbb;
         q = p->head.get();
         mbb += q->mbb;
-        while (likely(q->next)) {
+        while (likely(q->next))
+        {
           q = q->next.get();
           mbb += q->mbb;
         }
@@ -686,12 +835,14 @@ public:
       }
     }
 
-    if (size() > REBUILD_THRE * n_at_build){
+    if (size() > REBUILD_THRE * n_at_build)
+    {
       rebuild();
-    } 
+    }
   }
 
-  void rebuild(){
+  void rebuild()
+  {
     std::stack<PRTreeNode<T, B, D> *> sta;
     PRTreeNode<T, B, D> *p, *q;
     size_t length = idx2bb.size();
@@ -704,20 +855,27 @@ public:
 
     int i = 0;
     sta.push(root.get());
-    while(unlikely(!sta.empty())){
+    while (unlikely(!sta.empty()))
+    {
       p = sta.top();
       sta.pop();
 
-      if (unlikely(p->leaf)){
-        for (const auto &datum : p->leaf->data){
+      if (unlikely(p->leaf))
+      {
+        for (const auto &datum : p->leaf->data)
+        {
           new (b + i) DataType<T, D>{datum.first, datum.second};
           i++;
         }
-      } else {
-        if (likely(p->head)){
+      }
+      else
+      {
+        if (likely(p->head))
+        {
           q = p->head.get();
           sta.push(q);
-          while (likely(q->next)){
+          while (likely(q->next))
+          {
             q = q->next.get();
             sta.push(q);
           }
@@ -730,7 +888,8 @@ public:
   }
 
   template <class iterator>
-  void build(iterator &b, iterator &e, void *placement) {
+  void build(iterator &b, iterator &e, void *placement)
+  {
     n_at_build = size();
     vec<std::unique_ptr<PRTreeNode<T, B, D>>> prev_nodes;
     std::unique_ptr<PRTreeNode<T, B, D>> p, q, r;
@@ -738,12 +897,14 @@ public:
     auto first_tree = PseudoPRTree<T, B, D>(b, e);
     auto first_leaves = first_tree.get_all_leaves(e - b);
     parallel_for_each(first_leaves.begin(), first_leaves.end(), prev_nodes,
-                      [&](auto &leaf, auto &o) {
+                      [&](auto &leaf, auto &o)
+                      {
                         auto pp = std::make_unique<PRTreeNode<T, B, D>>(leaf);
                         o.push_back(std::move(pp));
                       });
     auto [bb, ee] = first_tree.as_X(placement, e - b);
-    while (likely(prev_nodes.size() > 1)) {
+    while (likely(prev_nodes.size() > 1))
+    {
       auto tree = PseudoPRTree<int, B, D>(bb, ee);
       vec<Leaf<int, B, D> *> leaves = tree.get_all_leaves(ee - bb);
       auto leaves_size = leaves.size();
@@ -752,7 +913,8 @@ public:
       tmp_nodes.reserve(leaves_size);
 
       parallel_for_each(
-          leaves.begin(), leaves.end(), tmp_nodes, [&](auto &leaf, auto &o) {
+          leaves.begin(), leaves.end(), tmp_nodes, [&](auto &leaf, auto &o)
+          {
             int idx, jdx;
             int len = leaf->data.size();
             auto pp = std::make_unique<PRTreeNode<T, B, D>>(leaf->mbb);
@@ -770,8 +932,7 @@ public:
               o.push_back(std::move(pp));
             } else {
               throw std::runtime_error("what????");
-            }
-          });
+            } });
 
       leaves.clear();
       vec<Leaf<int, B, D> *>().swap(leaves);
@@ -785,7 +946,8 @@ public:
         ee = std::move(tmp.second);
       }
     }
-    if (unlikely(prev_nodes.size() != 1)) {
+    if (unlikely(prev_nodes.size() != 1))
+    {
       throw std::runtime_error("#roots is not 1.");
     }
     root = std::move(prev_nodes[0]);
@@ -793,58 +955,81 @@ public:
     vec<std::unique_ptr<PRTreeNode<T, B, D>>>().swap(prev_nodes);
   }
 
-  auto find_all(const py::array_t<float> &x) {
+  auto find_all(const py::array_t<float> &x)
+  {
     const auto &buff_info_x = x.request();
     const auto &ndim = buff_info_x.ndim;
     const auto &shape_x = buff_info_x.shape;
     bool is_point = false;
-    if (ndim == 1 && (!(shape_x[0] == 2 * D || shape_x[0] == D))) {
+    if (ndim == 1 && (!(shape_x[0] == 2 * D || shape_x[0] == D)))
+    {
       throw std::runtime_error("Invalid Bounding box size");
-    } else if (ndim == 2 && (!(shape_x[1] == 2 * D || shape_x[1] == D))) {
+    }
+    else if (ndim == 2 && (!(shape_x[1] == 2 * D || shape_x[1] == D)))
+    {
       throw std::runtime_error(
           "Bounding box must have the shape (length, 2 * dim)");
-    } else if (ndim > 3) {
+    }
+    else if (ndim > 3)
+    {
       throw std::runtime_error("invalid shape");
     }
 
-    if (ndim == 1){
-      if (shape_x[0] == D){
-        is_point = true; 
+    if (ndim == 1)
+    {
+      if (shape_x[0] == D)
+      {
+        is_point = true;
       }
-    } else {
-      if (shape_x[1] == D){
-        is_point = true; 
+    }
+    else
+    {
+      if (shape_x[1] == D)
+      {
+        is_point = true;
       }
     }
     vec<BB<D>> X;
     X.reserve(ndim == 1 ? 1 : shape_x[0]);
     BB<D> bb;
-    if (ndim == 1) {
+    if (ndim == 1)
+    {
       {
         std::array<Real, D> minima;
         std::array<Real, D> maxima;
-        for (int i = 0; i < D; ++i) {
+        for (int i = 0; i < D; ++i)
+        {
           minima[i] = *x.data(i);
-          if (is_point){
+          if (is_point)
+          {
             maxima[i] = minima[i];
-          } else {
+          }
+          else
+          {
             maxima[i] = *x.data(i + D);
           }
         }
         bb = BB<D>(minima, maxima);
       }
       X.push_back(std::move(bb));
-    } else {
+    }
+    else
+    {
       X.reserve(shape_x[0]);
-      for (long int i = 0; i < shape_x[0]; i++) {
+      for (long int i = 0; i < shape_x[0]; i++)
+      {
         {
           std::array<Real, D> minima;
           std::array<Real, D> maxima;
-          for (int j = 0; j < D; ++j) {
+          for (int j = 0; j < D; ++j)
+          {
             minima[j] = *x.data(i, j);
-            if (is_point){
+            if (is_point)
+            {
               maxima[j] = minima[j];
-            } else {
+            }
+            else
+            {
               maxima[j] = *x.data(i, j + D);
             }
           }
@@ -857,25 +1042,33 @@ public:
     vec<vec<T>> out;
     out.reserve(length);
     parallel_for_each(X.begin(), X.end(), out,
-                      [&](const BB<D> &x, auto &o) { o.push_back(find(x)); });
+                      [&](const BB<D> &x, auto &o)
+                      { o.push_back(find(x)); });
     return out;
   }
 
-  auto find_one(const vec<float> &x) {
+  auto find_one(const vec<float> &x)
+  {
     bool is_point = false;
-    if (unlikely(!(x.size() == 2 * D || x.size() == D))) {
+    if (unlikely(!(x.size() == 2 * D || x.size() == D)))
+    {
       throw std::runtime_error("invalid shape");
     }
     std::array<Real, D> minima;
     std::array<Real, D> maxima;
-    if (x.size() == D){
+    if (x.size() == D)
+    {
       is_point = true;
     }
-    for (int i = 0; i < D; ++i) {
+    for (int i = 0; i < D; ++i)
+    {
       minima[i] = x.at(i);
-      if (is_point){
+      if (is_point)
+      {
         maxima[i] = minima[i];
-      } else {
+      }
+      else
+      {
         maxima[i] = x.at(i + D);
       }
     }
@@ -884,29 +1077,38 @@ public:
     return out;
   }
 
-  vec<T> find(const BB<D> &target) {
+  vec<T> find(const BB<D> &target)
+  {
     vec<T> out;
     queue<PRTreeNode<T, B, D> *> que;
     PRTreeNode<T, B, D> *p, *q;
-    auto qpush_if_intersect = [&](PRTreeNode<T, B, D> *r) {
-      if (unlikely((*r)(target))) {
+    auto qpush_if_intersect = [&](PRTreeNode<T, B, D> *r)
+    {
+      if (unlikely((*r)(target)))
+      {
         que.emplace(r);
       }
     };
 
     p = root.get();
     qpush_if_intersect(p);
-    while (likely(!que.empty())) {
+    while (likely(!que.empty()))
+    {
       p = que.front();
       que.pop();
 
-      if (unlikely(p->leaf)) {
+      if (unlikely(p->leaf))
+      {
         (*p->leaf)(target, out);
-      } else {
-        if (likely(p->head)) {
+      }
+      else
+      {
+        if (likely(p->head))
+        {
           q = p->head.get();
           qpush_if_intersect(q);
-          while (likely(q->next)) {
+          while (likely(q->next))
+          {
             q = q->next.get();
             qpush_if_intersect(q);
           }
@@ -916,33 +1118,43 @@ public:
     return out;
   }
 
-  void erase(const T idx) {
+  void erase(const T idx)
+  {
     auto it = idx2bb.find(idx);
-    if (unlikely(it == idx2bb.end())) {
+    if (unlikely(it == idx2bb.end()))
+    {
       throw std::runtime_error("Given index is not found.");
     }
     BB<D> target = it->second;
     queue<PRTreeNode<T, B, D> *> que;
     PRTreeNode<T, B, D> *p, *q;
-    auto qpush_if_intersect = [&](PRTreeNode<T, B, D> *r) {
-      if (unlikely((*r)(target))) {
+    auto qpush_if_intersect = [&](PRTreeNode<T, B, D> *r)
+    {
+      if (unlikely((*r)(target)))
+      {
         que.emplace(r);
       }
     };
 
     p = root.get();
     qpush_if_intersect(p);
-    while (likely(!que.empty())) {
+    while (likely(!que.empty()))
+    {
       p = que.front();
       que.pop();
 
-      if (unlikely(p->leaf)) {
+      if (unlikely(p->leaf))
+      {
         p->leaf->del(idx, target);
-      } else {
-        if (likely(p->head)) {
+      }
+      else
+      {
+        if (likely(p->head))
+        {
           q = p->head.get();
           qpush_if_intersect(q);
-          while (likely(q->next)) {
+          while (likely(q->next))
+          {
             q = q->next.get();
             qpush_if_intersect(q);
           }
@@ -952,12 +1164,14 @@ public:
 
     idx2bb.erase(idx);
     idx2data.erase(idx);
-    if (unlikely(REBUILD_THRE * size() < n_at_build)){
+    if (unlikely(REBUILD_THRE * size() < n_at_build))
+    {
       rebuild();
     }
   }
 
-  int64_t size(){
+  int64_t size()
+  {
     return static_cast<int64_t>(idx2bb.size());
   }
 };
